/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// logging extension  for pan
// (c) 1999 Jason Leach
// released under the GNU GPL

// key:
// ~ = partially coded
// ! = fully coded (I believe), but not tested
// * = finished and tested

// necessary globals:
//
// [ ] int logging_on - is it on or off? toggle
// [*] char* log_dir - path to logfile... should be in ~/.pan/data or so...

// necessary functions:
//
// [*] log_open - open the logfile
// [ ] log_close - close the logfile
// [*] log_add - add an entry to end of log, w/timestamp and such
// [~] log_view - Gtk+ window with log info pops up
// [ ] log_create - create the log file
// [~] logging_toggle - to be called from prefs screen

// extras (add last if at all):
//
// [ ] ability to choose logging options (downloaded files/connections/group updates)
// [ ] ability to limit max log file size
// [ ] colored log viewing options
// [ ] log viewer editing options (remove selected...etc)

#include <gnome.h>
#include "pan.h"
#include "log.h"
#include <time.h>
#include <string.h>

logger Pan_log = { 0, 1 };

//--------------------------------------------------------------------
// open a log file, return it as a FILE*
//--------------------------------------------------------------------
// FIXME: make use of the *gdata, logging should be per-group
int
log_open (void)
{
	char path[PATH_MAX];
	FILE *fp;
		
	g_snprintf(path, PATH_MAX + 1, "/%s/%s", props_get_log_dir(), Pan.active_server->name);
	
	if ((fp = fopen(path, "r")) != NULL) {
		Pan_log.file_open = 1;
		fclose(fp);
	} else
		pan_error_dialog("We couldn't open the log file at %s", path);
	return 0;
}


//
// close a log file
//
int
log_close (FILE *fp)
{
	char path[PATH_MAX];
		
	g_snprintf(path, PATH_MAX + 1, "%s/%s", props_get_log_dir(),
		   Pan.active_server->name);

	if (fclose(fp) != 0)
		pan_error_dialog("Error closing the log file at %s", path);
	return 0;
}


//--------------------------------------------------------------------
// remove CRLF (newline) from a string
//--------------------------------------------------------------------
void
remove_crlf (gchar *msg)
{
	char *tmp;
	if (!msg)
		return;

	while ((tmp = rindex(msg, '\n')) || (tmp = rindex(msg, '\r')))
		*tmp = 0;
}


//--------------------------------------------------------------------
// add a message to the log file with a variable argument list
//--------------------------------------------------------------------
// FIXME: regonize per-group logging
void
log_add_va (gchar *format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail(format != NULL);

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);

	log_add(msg);
}


//--------------------------------------------------------------------
// add a message to the log file
//--------------------------------------------------------------------
void
log_add (gchar *msg)
{
	time_t cur_time;
	char log_entry[256];
	char path[PATH_MAX];		// path to log file

	FILE *fp;					// file output

	time(&cur_time);

	g_snprintf(path, 256, "%s/%s", props_get_log_dir(),
		   Pan.active_server->name);

	if ((fp = fopen(path, "a")) != NULL) {
		Pan_log.file_open = 1;

		g_snprintf (log_entry, 256, "%s", ctime(&cur_time));
		remove_crlf (log_entry);

		strcat (log_entry, " >> ");
		strcat (log_entry, msg);

		fprintf (fp, "%s\n", log_entry);
		fclose (fp);
	} else
		pan_error_dialog ("We couldn't open the log file at %s", path);

	//  g_print ("%s", log_entry);
	//  g_free (cur_time);
	//g_free (log_entry);
}


//--------------------------------------------------------------------
// pop open a log viewer window, use a generic scrolling text window
//--------------------------------------------------------------------
void
log_view (void)
{
	GtkWidget *window;
	GtkWidget *text_box;

	if (!Pan_log.file_open) {
		log_open();
		return;
	}
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	text_box = gtk_text_new(NULL, NULL);

	gtk_container_add(GTK_CONTAINER(window), text_box);
	gtk_widget_show(window);
}


/*
void
logging_toggle (void)
{
	Pan.active_group->logging = Pan.active_group->logging ? 0 : 1;
	(Pan_log.logging_on) ? 0 : 1;
}
*/
